import airflow.models
from airflow.version import version as AIRFLOW_VERSION

from pkg_resources import parse_version

if parse_version(AIRFLOW_VERSION) >= parse_version("1.10.11"):
    from airflow import LoggingMixin
else:
    # Corrects path of import for Airflow versions below 1.10.11
    from airflow.utils.log.logging_mixin import LoggingMixin


class DAG(airflow.models.DAG, LoggingMixin):
    def __init__(self, *args, **kwargs):
        self.log.debug("odd dag starting")
        super().__init__(*args, **kwargs)

    def create_dagrun(self, *args, **kwargs):
        # run Airflow's create_dagrun() first
        self.log.debug(f"create_dagrun({args}, {kwargs})")
        dagrun = super(DAG, self).create_dagrun(*args, **kwargs)

        #create_dag_start_ms = self._now_ms()
        try:
            # send dagrun to ingestion
            self._send_dagrun(dagrun)
        except Exception as e:
            self.log.error(
                f'Failed to record metadata: {e} '
                f'{self._timed_log_message(create_dag_start_ms)}',
                exc_info=True
            )

        return dagrun

    def handle_callback(self, *args, **kwargs):
        self.log.debug(f"handle_callback({args}, {kwargs})")
        try:
            dagrun = args[0]
            self.log.debug(f"handle_callback() dagrun : {dagrun}")
            print("HERE! handle_callback")
            # self._report_task_instances(
            #     dagrun,
            #     kwargs.get('session')
            # )
        except Exception as e:
            self.log.error(
                f'Failed to record dagrun callback: {e} '
                f'dag_id={self.dag_id}',
                exc_info=True
            )

        return super().handle_callback(*args)

    def _send_dagrun(self, dagrun):
        # Собираем и валидируем данные
        print("VALIDATION")
        # Конвертим их в нужный формат (см. одд спеку)
        print("CONVERTION")
        # Отсылаем на ingest через try/except
        print("SEND TO INGESTION")



from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = [
    "attrs>=19.3",
    "requests>=2.24.0",
    "sqlparse==0.4.1",
]

extras_require = {
    "tests": [
        "pytest",
        "pytest-cov",
        "mock",
        "flake8",
        "SQLAlchemy==1.3.24",       # must be set to 1.3.* for airflow tests compatibility
        "Flask-SQLAlchemy==2.4.4",  # must be set to 2.4.* for airflow tests compatibility
        "pandas-gbq==0.14.1",       # must be set to 0.14.* for airflow tests compatibility
        "apache-airflow==1.10.12",
        "apache-airflow[gcp_api]==1.10.12",
        "apache-airflow[google]==1.10.12",
        "apache-airflow[postgres]==1.10.12",
        "snowflake-connector-python==2.4.3",
    ],
}
extras_require["dev"] = set(sum(extras_require.values(), []))

setup(
    name="odd_airflow",
    version="0.0.3",
    description="ODD adapter to Airflow",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Provectus ODD team",
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    python_requires=">=3.6",
    zip_safe=False,
    keywords="odd-airflow",
)